from confluence.models.contentbody import ContentBody
from confluence.models.contenthistory import ContentHistory
from confluence.models.version import Version
from confluence.models.space import Space
from enum import Enum
import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ContentType(Enum):
    """
    The set of valid content types in confluence along with their representation on the API

    https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/api/model/content/ContentType.html
    """
    ATTACHMENT = "attachment"
    BLOG_POST = "blogpost"
    COMMENT = "comment"
    PAGE = "page"


class ContentStatus(Enum):
    """
    The set of valid content statuses in confluence along with their API representation.

    https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/api/model/content/ContentStatus.html
    """
    CURRENT = "current"
    DRAFT = "draft"
    HISTORICAL = "historical"
    TRASHED = "trashed"


class CommentLocation(Enum):
    INLINE = 'inline'
    FOOTER = 'footer'
    RESOLVED = 'resolved'


class CommentDepth(Enum):
    ROOT = ''
    ALL = 'all'


class Content:
    """
    Main content class for all the different content types. This includes
    pages, blogs, comments and attachments.

    The type field will allow the end user to distinguish between the types
    from calling code.
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.id = json['id']  # type: str
        self.title = json['title']  # type: str
        self.status = ContentStatus(json['status'])  # type: ContentStatus
        self.type = ContentType(json['type'])  # type: ContentType

        if 'metadata' in json:
            self.metadata = json['metadata']  # type: Dict[str, Any]

        if 'extensions' in json:
            self.extensions = json['extensions']  # type: Dict[str, Any]

        if 'space' in json:
            self.space = Space(json['space'])

        if 'body' in json:
            self.body = ContentBody(json['body'])

        if 'history' in json:
            self.history = ContentHistory(json['history'])

        if 'version' in json:
            self.version = Version(json['version'])

    def __str__(self):
        return '{} - {}'.format(self.id, self.title)



from confluence.models.auditrecord import AuditRecord
import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def test_create_audit_record(): 
    a = AuditRecord({
        "author":  {
            "type":  "user",
            "displayName": "A Name",
            "username": "an",
            "userKey": "1"
        },
        "remoteAddress": "1.1.1.1,2.2.2.2",
        "creationDate": 1517497826248,
        "summary": "Space Workflow States Initialized",
        "description": "",
        "category": "Comala Workflows",
        "sysAdmin": True,
        "affectedObject": {
            "name": "About",
            "objectType": "Space"
        },
        "changedValues": [{
            "name": "stateName",
            "oldValue": "",
            "newValue": "Up to date"
        }, {
            "name": "overrideCurrentState",
            "oldValue": "",
            "newValue": "false"
        }],
        "associatedObjects": []
    })

    assert str(a) == 'Space Workflow States Initialized'
    assert '1.1.1.1' in a.remote_address and '2.2.2.2' in a.remote_address
    assert len(a.changed_values) == 2
    assert a.creation_date.strftime('%Y-%m-%d') == '2018-02-01'



from confluence.models.version import Version
import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def test_page_update_creation():
    cv = Version({
        'by': {
            'username': '1',
            'displayName': '2',
            'userKey': '3',
            'type': '4'
        },
        'when': '2017-02-01',
        'message': 'Hello',
        'number': 1,
        'minorEdit': False,
        'hidden': False
    })

    assert str(cv) == '1'



import logging
import requests
from typing import Dict

from confluence.exceptions.generalerror import ConfluenceError

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ConfluenceValueTooLong(ConfluenceError):
    """
    This exception corresponds to 413 errors on the REST API
    """
    def __init__(self, path, params, response):
        # type: (str, Dict[str, str], requests.Response) -> None
        msg = 'Resource post to path {} was too large'.format(path)
        super(ConfluenceValueTooLong, self).__init__(path, params, response, msg)



from confluence.models.version import Version
from confluence.models.user import User
import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ContentHistory:
    """
    Represents the history of a piece of content(blog|page|comment|attachment)
    in confluence.
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.latest = json['latest']
        self.author = User(json['createdBy'])
        self.created_date = json['createdDate']

        # Fields only returned if the history.lastUpdated is expanded
        if 'lastUpdated' in json:
            self.last_updated = Version(json['lastUpdated'])

        if 'previousVersion' in json:
            self.previous_version = Version(json['previousVersion'])

        if 'nextVersion' in json:
            self.next_version = Version(json['nextVersion'])

        if 'contributors' in json:
            # Note: this is not properly implemented yet, we don't turn this
            # into objects with known properties.
            self.contributors = json['contributors']



import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Label:
    """
    Represents a label on a piece of confluence content. c.f.
    https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/labels/Label.html
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.id = json['id']  # type: str
        self.name = json['name']  # type: str
        self.prefix = json['prefix']  # type: str

    def __str__(self):
        return self.name



import logging
from typing import Any, Dict, List

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class TaskName:
    """
    Represents the name of a long running task.
    """
    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.key = json['key']  # type: str
        self.args = json['args']  # type: List[str]

    def __str__(self):
        return self.key


class TaskMessage:
    """
    Represents a message about a long running task.
    """
    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.translation = json['translation']  # type: str
        self.args = json['args']  # type: List[str]

    def __str__(self):
        return self.translation


class LongTask:
    """
    Represents a single long running task in confluence (e.g. PDF space export)
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.id = json['id']  # type: str
        self.name = TaskName(json['name'])  # type: TaskName
        self.elapsed_time = json['elapsedTime']  # type: int
        self.percentage_complete = json['percentageComplete']  # type: int
        self.successful = json['successful']  # type: bool
        self.messages = [TaskMessage(m) for m in json['messages']]  # type: List[TaskMessage]

    def __str__(self):
        return str(self.name)



from confluence.models.longtask import LongTask
import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def test_create_full_json():
    t = LongTask({
        "id": "14365eab-f2df-4ecb-9458-75ad3af903a7",
        "name": {
            "key": "com.atlassian.confluence.extra.flyingpdf.exporttaskname",
            "args": []},
        "elapsedTime": 101770,
        "percentageComplete": 100,
        "successful": True,
        "messages": [{
            "translation": "Finished PDF space export. Download <a href=\"/confluence/download/temp/pdfexport-20180111-110118-1209-23/x-110118-1209-24.pdf\">here</a>.",
            "args": []
        }]
    })

    assert str(t) == 'com.atlassian.confluence.extra.flyingpdf.exporttaskname'
    assert len(t.messages) == 1
    assert t.successful



import logging
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Group:
    """
    Represents a group object in confluence.
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.type = json['type']
        self.name = json['name']

    def __str__(self):
        return self.name
